/*=========================================================================
| (c) 2011  Total Phase, Inc.
|--------------------------------------------------------------------------
| Project : Komodo Examples
| File    : async.c
|--------------------------------------------------------------------------
| Uses the asynchronous interface of the Komodo CAN Duo.
|--------------------------------------------------------------------------
| Redistribution and use of this file in source and binary forms, with
| or without modification, are permitted.
|
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
| "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
| LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
| FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
| COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
| INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
| BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
| LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
| ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
| POSSIBILITY OF SUCH DAMAGE.
 ========================================================================*/

/*=========================================================================
| INCLUDES
 ========================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "komodo.h"


/*=========================================================================
| USAGE INFORMATION
 ========================================================================*/
static void print_usage (void) {
    printf(
"Usage: async PORT TARGET_PWR CAN_ID ITER\n"
"\n"
"This utility uses the asynchronous interface to send packets on CAN A.\n"
"\n"
"  PORT       : Komodo port\n"
"  TARGET_PWR : 1 turns on target power, 0 does not\n"
"  CAN_ID     : Packets are broadcast with this ID\n"
"  ITER       : Number of packets to process asynchronously.\n"
"\n"
"For product documentation and specifications, see www.totalphase.com.\n");
};


/*=========================================================================
| FUNCTIONS
 ========================================================================*/
static void async (Komodo km, int can_id, km_can_ch_t can_ch, int iter) {
    int i          = 0;
    int ret        = 0;
    int timeout    = 500;
    u08 data[]     = { 1, 2, 3, 4, 5, 6, 7 };
    int data_len   = 7;
    u32 arb_count  = 0;

    km_can_packet_t pkt;
    pkt.remote_req   = 0;
    pkt .extend_addr = 0;
    pkt.dlc          = data_len;
    pkt.id           = can_id;

    // Enable Komodo
    ret = km_enable(km);
    if (ret != KM_OK) {
        printf("Unable to enable Komodo\n");
        return;
    }

    // Send first packet
    km_can_async_submit(km, can_ch, 0, &pkt, data_len, data);
    printf("Submitted packet 1\n");

    for (i = 0; i < (iter - 1); i++) {
        // Send another packet while the first is still in progress
        km_can_async_submit(km, can_ch, 0, &pkt, data_len, data);
        printf("Submitted packet %d\n", i + 2);

        // The application can now perform other functions
        // while the Komodo is sending the previous packet
        km_sleep_ms(100);

        // Collect response to previously submitted packet
        ret = km_can_async_collect(km, timeout, &arb_count);

        if (ret == KM_OK) printf("Collected packet %d\n", i + 1);
        else              printf("Error %d\n", ret);
    }

    // Collect the last response
    ret = km_can_async_collect(km, timeout, &arb_count);
    if (ret == KM_OK) printf("Collected packet %d\n\n", i + 1);
    else              printf("Error %d\n\n", ret);
}


/*=========================================================================
| MAIN PROGRAM
 ========================================================================*/
int main (int argc, char *argv[]) {
    Komodo km;
    int    ret;
    int    iter;
    int    port;
    int    power;
    int    can_id;
    u32    bitrate = 125000; // Hz

    if (argc < 5) {
        print_usage();
        return 1;
    }

    port   = atoi(argv[1]);
    power  = atoi(argv[2]) ? KM_TARGET_POWER_ON : KM_TARGET_POWER_OFF;
    can_id = strstr(argv[3], "0x") ? strtol(argv[3], 0, 16) : atoi(argv[3]);
    iter   = atoi(argv[4]);

    // Open the interface
    km = km_open(port);
    if (km <= 0) {
        printf("Unable to open Komodo on port %d\n", port);
        printf("Error code = %d\n", km);
        return 1;
    }

    // Acquire features
    ret = km_acquire(km, KM_FEATURE_CAN_A_CONFIG | KM_FEATURE_CAN_A_CONTROL);
    printf("Acquired features 0x%x\n", ret);

    // Set bitrate
    ret = km_can_bitrate(km, KM_CAN_CH_A, bitrate);
    printf("Bitrate set to %d kHz\n", ret/1000);

    // Set target power
    km_can_target_power(km, KM_CAN_CH_A, power);
    printf("Target power %s\n\n", power ? "ON" : "OFF");

    async(km, can_id, KM_CAN_CH_A, iter);

    // Close and exit
    km_close(km);
    return 0;
}
