/*=========================================================================
| (c) 2011  Total Phase, Inc.
|--------------------------------------------------------------------------
| Project : Komodo Examples
| File    : gpio.c
|--------------------------------------------------------------------------
| Performs some simple GPIO operations with a Komodo interface.
|--------------------------------------------------------------------------
| Redistribution and use of this file in source and binary forms, with
| or without modification, are permitted.
|
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
| "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
| LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
| FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
| COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
| INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
| BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
| LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
| ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
| POSSIBILITY OF SUCH DAMAGE.
 ========================================================================*/

/*=========================================================================
| INCLUDES
 ========================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "komodo.h"

#define NUM_GPIOS 8


/*=========================================================================
| USAGE INFORMATION
 ========================================================================*/
static void print_usage (void) {
    printf(
"Usage: gpio PORT\n"
"\n"
"Example utility that demonstrates GPIO functions of the Komodo CAN Duo.\n"
"\n"
"  PORT: Komodo port\n"
"\n"
"For product documentation and specifications, see www.totalphase.com.\n");
};


/*=========================================================================
| FUNCTIONS
 ========================================================================*/
static void show_pin_state (int state, int pin) {
    int mask = (1 << (pin - 1));
    printf("Pin %d is %s\n", pin, state & mask ? "high" : "low");
}

static void monitor_changes (Komodo km) {
    int i;
    int oldval;
    time_t endtime;
    int wait_sec = 20;

    printf("\nConfigure pins as inputs with pullups\n");
    for (i = 0; i < NUM_GPIOS; i++)
        km_gpio_config_in(km, i, KM_PIN_BIAS_PULLUP, KM_PIN_TRIGGER_NONE);

    // Enable Komodo
    km_enable(km);
    printf("Monitoring GPIO pin changes for %d seconds:\n\n", wait_sec);

    endtime = time(NULL) + wait_sec;
    oldval  = km_gpio_get(km);

    while (time(NULL) < endtime) {
        int newval, change;

        // Check for pin changes
        newval = km_gpio_get(km);
        if (newval != oldval) {
            change = oldval ^ newval;

            // Display the pins that changed
            for (i = 0; i < NUM_GPIOS; i++)
                if (change & (1 << i))
                    show_pin_state(newval, i + 1);

            // Save the current state
            oldval = newval;
        }
    }
    printf("\n");
}

static void gpio (Komodo km) {
    int ret;

    printf("Configuring Pin 1 as an output\n");
    km_gpio_config_out(km, KM_GPIO_PIN_1_CONFIG, KM_PIN_DRIVE_NORMAL,
                       KM_PIN_SRC_SOFTWARE_CTL);

    printf("Configuring Pin 2 as an input with pullup\n");
    km_gpio_config_in(km, KM_GPIO_PIN_2_CONFIG, KM_PIN_BIAS_PULLUP,
                      KM_PIN_TRIGGER_NONE);

    // Enable Komodo
    km_enable(km);

    printf("\nSetting Pin 1 to logic low\n");
    km_gpio_set(km, 0, KM_GPIO_PIN_1_MASK);

    // Show the state of each pin
    // Pin 1 should be low because it was set low
    // Pin 2 should be high because it's an input with a pullup resistor.
    ret = km_gpio_get(km);
    show_pin_state(ret, 1);
    show_pin_state(ret, 2);

    // Disable the Komodo interface to change the GPIO configuration
    km_disable(km);

    printf("\nConfiguring Pin 2 as an input with pulldown\n");
    km_gpio_config_in(km, KM_GPIO_PIN_2_CONFIG, KM_PIN_BIAS_PULLDOWN,
                      KM_PIN_TRIGGER_NONE);

    // Enable and read value of PIN 2
    km_enable(km);
    ret = km_gpio_get(km);
    show_pin_state(ret, 2);

    km_disable(km);

    // Now, set all GPIOs as inputs and monitor for changes
    monitor_changes(km);
}


/*=========================================================================
| MAIN PROGRAM
 ========================================================================*/
int main (int argc, char *argv[]) {
    Komodo km;
    int    ret;
    int    port;

    if (argc < 2) {
        print_usage();
        return(1);
    }

    port   = atoi(argv[1]);

    // Open the interface
    km = km_open(port);
    if (km <= 0) {
        printf("Unable to open Komodo on port %d\n", port);
        printf("Error code = %d\n", km);
        return 1;
    }

    // Acquire features
    ret = km_acquire(km, KM_FEATURE_GPIO_LISTEN  |
                         KM_FEATURE_GPIO_CONTROL |
                         KM_FEATURE_GPIO_CONFIG);
    printf("Acquired features 0x%x\n", ret);

    gpio(km);

    // Close and exit
    km_close(km);
    return 0;
}
