/*=========================================================================
| (c) 2011  Total Phase, Inc.
|--------------------------------------------------------------------------
| Project : Komodo Examples
| File    : loopback.c
|--------------------------------------------------------------------------
| Demonstrates how to open ports, acquire features, write and read data.
|--------------------------------------------------------------------------
| Redistribution and use of this file in source and binary forms, with
| or without modification, are permitted.
|
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
| "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
| LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
| FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
| COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
| INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
| BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
| LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
| ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
| POSSIBILITY OF SUCH DAMAGE.
 ========================================================================*/

/*=========================================================================
| INCLUDES
 ========================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "komodo.h"


/*=========================================================================
| USAGE INFORMATION
 ========================================================================*/
static void print_usage (void) {
    printf(
"Usage: loopback PORT1 PORT2\n"
"\n"
"Example utility showing how to read and write CAN data.\n"
"CAN channels A and B must be connected with a terminated cable.\n"
"PORT1 and PORT2 may be the same or different.\n"
"\n"
"  PORT1: Komodo port that connects to CAN channel A\n"
"  PORT2: Komodo port that connects to CAN channel B\n"
"\n"
"For product documentation and specifications, see www.totalphase.com.\n");
};


/*=========================================================================
| FUNCTIONS
 ========================================================================*/
static void print_num_array (u08* array, int len) {
    int i = 0;

    printf("[ ");
    for (i = 0; i < len; i++)
        printf("%x ", array[i]);
    printf("]");
}

static void loopback (Komodo kma, Komodo kmb) {
    int i;
    int ret;
    u32 can_id     = 0xFF;
    u08 data_out[] = { 1, 2, 3, 4, 5, 6, 7, 8 };
    int data_len   = 8;
    u08 data_in[8];
    u32 arb_count[8];

    km_can_info_t   info;
    km_can_packet_t pkt;
    pkt.remote_req  = 0;
    pkt.extend_addr = 0;
    pkt.dlc         = data_len;
    pkt.id          = can_id;

    // Enable CAN A and B
    ret = km_enable(kma);
    if (ret != KM_OK) {
        printf("Unable to enable Komodo A\n");
        return;
    }

    if (kma != kmb) {
        ret = km_enable(kmb);
        if (ret != KM_OK) {
            printf("Unable to enable Komodo B\n");
            return;
        }
    }

    // Send data on CAN A
    km_can_write(kma, KM_CAN_CH_A, 0, &pkt, data_len, data_out, arb_count);

    printf("Sent data:     ");
    print_num_array(data_out, data_len);

    // Read data on CAN B
    do {
        ret = km_can_read(kmb, &info, &pkt, data_len, data_in);
    } while(ret == 0);

    printf("\nReceived data: ");
    print_num_array(data_in, ret);

    // Verify received data
    for (i = 0; i < data_len; i++)
        if (data_out[i] != data_in[i])
            break;

    printf("\nVerifying data... %s\n\n", i == data_len ? "PASS" : "FAIL");
}


/*=========================================================================
| MAIN PROGRAM
 ========================================================================*/
int main (int argc, char *argv[]) {
    Komodo kma, kmb;
    int    reta, retb;
    int    porta, portb;
    u16    timeout = 1000;   // ms
    u32    bitrate = 125000; // Hz

    if (argc < 3) {
        print_usage();
        return 1;
    }

    porta = atoi(argv[1]);
    portb = atoi(argv[2]);

    // Connect to and configure
    kma = km_open(porta);
    if (kma <= 0) {
        printf("Unable to open Komodo on port %d\n", porta);
        printf("Error code = %d\n", kma);
        return 1;
    }

    if (porta != portb) {
        kmb = km_open(portb);
        if (kmb <= 0) {
            printf("Unable to open Komodo on port %d\n", portb);
            printf("Error code = %d\n", kmb);
            return 1;
        }
    }
    else kmb = kma;

    // Setup CAN channel A for writing and B for listening
    reta = km_acquire(kma, KM_FEATURE_CAN_A_CONFIG  |
                           KM_FEATURE_CAN_A_CONTROL);
    retb = km_acquire(kmb, KM_FEATURE_CAN_B_CONFIG  |
                           KM_FEATURE_CAN_B_CONTROL |
                           KM_FEATURE_CAN_B_LISTEN);
    printf("Features for CAN A: 0x%02x, CAN B: 0x%02x\n", reta, retb);

    // Set bitrate
    reta = km_can_bitrate(kma, KM_CAN_CH_A, bitrate);
    retb = km_can_bitrate(kmb, KM_CAN_CH_B, bitrate);
    printf("Bitrate for CAN A set to %d kHz, CAN B set to %d kHz\n",
           reta/1000, retb/1000);

    // Set timeout
    km_timeout(kmb, timeout);
    printf("Timeout for CAN B set to %d ms\n", timeout);

    // Set target power
    km_can_target_power(kma, KM_CAN_CH_A, KM_TARGET_POWER_ON);
    km_can_target_power(kmb, KM_CAN_CH_B, KM_TARGET_POWER_OFF);
    printf("Enabled target power for CAN A and disabled for CAN B\n\n");

    loopback(kma, kmb);

    // Close and exit
    km_close(kma);
    km_close(kmb);
    return 0;
}
