/*=========================================================================
| (c) 2011  Total Phase, Inc.
|--------------------------------------------------------------------------
| Project : Komodo Examples
| File    : request.c
|--------------------------------------------------------------------------
| Request data from specified CAN ID and print received data.
|--------------------------------------------------------------------------
| Redistribution and use of this file in source and binary forms, with
| or without modification, are permitted.
|
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
| "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
| LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
| FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
| COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
| INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
| BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
| LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
| ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
| POSSIBILITY OF SUCH DAMAGE.
 ========================================================================*/

/*=========================================================================
| INCLUDES
 ========================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "komodo.h"


/*=========================================================================
| USAGE INFORMATION
 ========================================================================*/
static void print_usage (void) {
    printf(
"Usage: request PORT TARGET_PWR CAN_ID \n"
"\n"
"Example utility which requests data from provided CAN ID on CAN A.\n"
"\n"
"  PORT: Komodo port\n"
"  TARGET_PWR: 1 turns on target power, 0 does not"
"  CAN_ID: ID to request CAN packet from\n"
"\n"
"For product documentation and specifications, see www.totalphase.com.\n");
};


/*=========================================================================
| FUNCTIONS
 ========================================================================*/
static void print_num_array (u08* array, int len) {
    int i = 0;

    printf("[ ");
    for (i = 0; i < len; i++)
        printf("%x ", array[i]);
    printf("]");
}

static void request (Komodo km, int can_id, int ch) {
    int    ret;
    int    data_len = 8;
    u08    data[8];
    u32    arb_count[8];

    km_can_info_t   info;
    km_can_packet_t pkt;
    pkt.remote_req  = 1;
    pkt.extend_addr = 0;
    pkt.dlc         = data_len;
    pkt.id          = can_id;

    // Enable Komodo
    ret = km_enable(km);
    if (ret != KM_OK) {
        printf("Unable to enable Komodo\n");
        return;
    }

    printf("Request data from CAN ID %d (0x%x)\n", can_id, can_id);

    // Send a remote frame with no data.  5th arg is 0 and
    // data array is not important.
    km_can_write(km, ch, 0, &pkt, 0, data, arb_count);

    // Look for packet with desired CAN ID. Store in data[].
    do {
        ret = km_can_read(km, &info, &pkt, data_len, data);
        if (info.status & KM_READ_TIMEOUT) {
            printf("Timeout: no data!\n");
            return;
        }
    } while (pkt.id != can_id);

    printf("Received %d byte(s): ", ret);

    print_num_array(data, ret);
    printf("\n");
}


/*=========================================================================
| MAIN PROGRAM
 ========================================================================*/
int main (int argc, char *argv[]) {
    Komodo km;
    int    ret;
    int    port;
    int    power;
    int    can_id;
    int    bitrate  = 125000; // kHz
    int    timeout  = 3000;   // ms

    if (argc < 4) {
        print_usage();
        return 1;
    }

    port   = atoi(argv[1]);
    power  = atoi(argv[2]) ? KM_TARGET_POWER_ON : KM_TARGET_POWER_OFF;
    can_id = strstr(argv[3], "0x") ? strtol(argv[3], 0, 16) : atoi(argv[3]);

    // Open the interface
    km = km_open(port);
    if (km <= 0) {
        printf("Unable to open Komodo on port %d\n", port);
        printf("Error code = %d\n", km);
        return 1;
    }

    // Acquire features to control and listen CAN A
    ret = km_acquire(km, KM_FEATURE_CAN_A_CONFIG  |
                         KM_FEATURE_CAN_A_CONTROL |
                         KM_FEATURE_CAN_A_LISTEN);
    printf("Acquired features: 0x%x\n", ret);

    // Set bitrate
    ret = km_can_bitrate(km, KM_CAN_CH_A, bitrate);
    printf("Bitrate set to %d kHz\n", ret/1000);

    // Set timeout
    km_timeout(km, timeout);
    printf("Timeout set to %d ms\n", timeout);

    // Set target power
    km_can_target_power(km, KM_CAN_CH_A, power);
    printf("Target power %s\n", power ? "ON" : "OFF");

    request(km, can_id, KM_CAN_CH_A);

    // Close and exit
    km_close(km);
    return 0;
}
