/*=========================================================================
| (c) 2011  Total Phase, Inc.
|--------------------------------------------------------------------------
| Project : Komodo Examples
| File    : async.cs
|--------------------------------------------------------------------------
| Uses the asynchronous interface of the Komodo CAN Duo.
|--------------------------------------------------------------------------
| Redistribution and use of this file in source and binary forms, with
| or without modification, are permitted.
|
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
| "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
| LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
| FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
| COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
| INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
| BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
| LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
| ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
| POSSIBILITY OF SUCH DAMAGE.
 ========================================================================*/
using System;
using System.IO;
using TotalPhase;

/*=========================================================================
| CLASS
 ========================================================================*/
public class async {

    /*=====================================================================
    | USAGE INFORMATION
    =====================================================================*/
    static void print_usage () {
        Console.Write(
"Usage: async PORT TARGET_PWR CAN_ID ITER\n" +
"\n" +
"This utility uses the asynchronous interface to send packets on CAN A.\n" +
"\n" +
"  PORT       : Komodo port\n" +
"  TARGET_PWR : 1 turns on target power, 0 does not\n" +
"  CAN_ID     : Packets are broadcast with this ID\n" +
"  ITER       : Number of packets to process asynchronously.\n" +
"\n" +
"For product documentation and specifications, see www.totalphase.com.\n");
        Console.Out.Flush();
    }


    /*=====================================================================
    | FUNCTIONS
     ====================================================================*/
    static void _async (int km, uint can_id, km_can_ch_t can_ch, int iter) {
        int    i         = 0;
        int    ret       = 0;
        uint   timeout   = 500;
        byte[] data      = { 1, 2, 3, 4, 5, 6, 7, 8 };
        byte   data_len  = (byte)data.Length;
        uint   arb_count = 0;

        KomodoApi.km_can_packet_t pkt;
        pkt.remote_req  = 0;
        pkt.extend_addr = 0;
        pkt.dlc         = data_len;
        pkt.id          = can_id;

        // Enable Komodo
        ret = KomodoApi.km_enable(km);
        if (ret != (int)km_status_t.KM_OK) {
            Console.Write("Unable to enable Komodo\n");
            return;
        }

        // Send first packet
        KomodoApi.km_can_async_submit(km, can_ch, 0, pkt, data_len, data);
        Console.Write("Submitted packet 1\n");

        for (i = 0; i < (iter - 1); i++) {
            // Send another packet while the first is still in progress
            KomodoApi.km_can_async_submit(km, can_ch, 0, pkt, data_len, data);
            Console.Write("Submitted packet {0}\n", i + 2);

            // The application can now perform other functions
            // while the Komodo is sending the previous packet
            KomodoApi.km_sleep_ms(100);

            // Collect response to previously submitted packet
            ret = KomodoApi.km_can_async_collect(km, timeout, ref arb_count);

            if (ret == (int)km_status_t.KM_OK)
                Console.Write("Collected packet {0}\n", i + 1);
            else
                Console.Write("Error {0}\n", ret);
        }

        // Collect the last response
        ret = KomodoApi.km_can_async_collect(km, timeout, ref arb_count);
        if (ret == (int)km_status_t.KM_OK)
            Console.Write("Collected packet {0}\n\n", i + 1);
        else
            Console.Write("Error {0}\n\n", ret);
    }


    /*=====================================================================
    | MAIN PROGRAM
    =====================================================================*/
    public static void Main (String[] args) {
        int        km;
        int        ret;
        int        iter;
        int        port;
        km_power_t power;
        uint       can_id;
        uint       bitrate = 125000; // Hz

        if (args.Length < 4) {
            print_usage();
            return;
        }

        port   = Convert.ToInt32(args[0]);
        power  = Convert.ToInt32(args[1]) == 1 ?
                 km_power_t.KM_TARGET_POWER_ON :
                 km_power_t.KM_TARGET_POWER_OFF;
        can_id = args[2].IndexOf("0x") == 0 ?
                 Convert.ToUInt32(args[2], 16) :
                 Convert.ToUInt32(args[2]);
        iter   = Convert.ToInt32(args[3]);

        // Open the interface
        km = KomodoApi.km_open(port);
        if (km <= 0) {
            Console.Write("Unable to open Komodo on port {0:d}\n", port);
            Console.Write("Error code = {0:d}\n", km);
            return;
        }

        // Acquire features
        ret = KomodoApi.km_acquire(km, KomodoApi.KM_FEATURE_CAN_A_CONFIG |
                                       KomodoApi.KM_FEATURE_CAN_A_CONTROL);
        Console.Write("Acquired features 0x{0:x}\n", ret);

        // Set bitrate
        ret = KomodoApi.km_can_bitrate(km, km_can_ch_t.KM_CAN_CH_A, bitrate);
        Console.Write("Bitrate set to {0:d} kHz\n", ret/1000);

        // Set target power
        KomodoApi.km_can_target_power(km,  km_can_ch_t.KM_CAN_CH_A, power);
        Console.Write("Target power {0:s}\n\n",
                      power == km_power_t.KM_TARGET_POWER_ON ? "ON" : "OFF");

        _async(km, can_id, km_can_ch_t.KM_CAN_CH_A, iter);

       //Close and exit
       KomodoApi.km_close(km);
       return;
    }
}
