/*=========================================================================
| (c) 2011  Total Phase, Inc.
|--------------------------------------------------------------------------
| Project : Komodo Examples
| File    : gpio.cs
|--------------------------------------------------------------------------
| Performs some simple GPIO operations with a Komodo interface.
|--------------------------------------------------------------------------
| Redistribution and use of this file in source and binary forms, with
| or without modification, are permitted.
|
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
| "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
| LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
| FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
| COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
| INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
| BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
| LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
| ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
| POSSIBILITY OF SUCH DAMAGE.
 ========================================================================*/
using System;
using System.IO;
using TotalPhase;

/*=========================================================================
| CLASS
 ========================================================================*/
public class gpio {
    const int  NUM_GPIOS = 8;


    /*=========================================================================
    | USAGE INFORMATION
     ========================================================================*/
    static void print_usage () {
        Console.Write(
"Usage: gpio PORT\n" +
"\n" +
"Example utility that demonstrates GPIO functions of the Komodo CAN Duo.\n" +
"\n" +
"  PORT: Komodo port\n" +
"\n" +
"For product documentation and specifications, see www.totalphase.com.\n");
        Console.Out.Flush();
}


    /*=====================================================================
    | FUNCTIONS
     ====================================================================*/
    static void show_pin_state (int state, int pin) {
        int mask = (1 << (pin - 1));
        Console.Write("Pin {0:d} is {1:s}\n", pin,
                      (state & mask) != 0 ? "high" : "low");
        Console.Out.Flush();
    }

    static void monitor_changes (int km) {
        int      i;
        int      oldval;
        DateTime endtime;
        const int wait_sec = 20;

        Console.Write("\nConfigure pins as inputs with pullups\n");
        for (i = 0; i < NUM_GPIOS; i++)
            KomodoApi.km_gpio_config_in(km, (byte)i,
                                   (byte)km_pin_bias_t.KM_PIN_BIAS_PULLUP,
                                   (byte)km_pin_trigger_t.KM_PIN_TRIGGER_NONE);

        // Enable Komodo
        KomodoApi.km_enable(km);
        Console.Write("Monitoring GPIO pin changes for {0} seconds:\n\n",
                      wait_sec);

        endtime = DateTime.Now.AddSeconds(wait_sec);
        oldval  = KomodoApi.km_gpio_get(km);

        while (DateTime.Now < endtime) {
            int newval, change;

            // Check for pin changes
            newval = KomodoApi.km_gpio_get(km);
            if (newval != oldval) {
                change = oldval ^ newval;

                // Display the pins that changes
                for (i = 0; i < NUM_GPIOS; i++)
                    if ((change & (1 << i)) != 0)
                        show_pin_state(newval, i + 1);

                // Save the current state
                oldval = newval;
            }
        }
        Console.Write("\n");
    }

    static void _gpio (int km) {
        int ret;

        Console.Write("Configuring Pin 1 as an output\n");
        KomodoApi.km_gpio_config_out(km, KomodoApi.KM_GPIO_PIN_1_CONFIG,
                                (byte)km_pin_drive_t.KM_PIN_DRIVE_NORMAL,
                                (byte)km_pin_source_t.KM_PIN_SRC_SOFTWARE_CTL);

        Console.Write("Configuring Pin 2 as an input with pullup\n");
        KomodoApi.km_gpio_config_in(km, KomodoApi.KM_GPIO_PIN_2_CONFIG,
                                 (byte)km_pin_bias_t.KM_PIN_BIAS_PULLUP,
                                 (byte)km_pin_trigger_t.KM_PIN_TRIGGER_NONE);

        // Enable Komodo
        KomodoApi.km_enable(km);

        // Set output low
        Console.Write("\nSetting Pin 1 to logic low\n");
        KomodoApi.km_gpio_set(km, 0, KomodoApi.KM_GPIO_PIN_1_MASK);

        // Show the state of each pin
        // Pin 1 should be low because it was set it low.
        // Pin 2 should be high because it's an input with a pullup resistor.
        ret = KomodoApi.km_gpio_get(km);
        show_pin_state(ret, 1);
        show_pin_state(ret, 2);

        // Disable the Komodo interface to change the GPIO configuration
        KomodoApi.km_disable(km);

        Console.Write("\nConfiguring Pin 2 as an input with pulldown\n");
        KomodoApi.km_gpio_config_in(km, KomodoApi.KM_GPIO_PIN_2_CONFIG,
                                   (byte)km_pin_bias_t.KM_PIN_BIAS_PULLDOWN,
                                   (byte)km_pin_trigger_t.KM_PIN_TRIGGER_NONE);

        // Enable and read value of PIN 2
        KomodoApi.km_enable(km);
        ret = KomodoApi.km_gpio_get(km);
        show_pin_state(ret, 2);

        KomodoApi.km_disable(km);

        // Now, set all GPIOs as inputs and monitor for changes
        monitor_changes(km);
    }


    /*=====================================================================
    | MAIN PROGRAM
    =====================================================================*/
    public static void Main (String[] args) {
        int km;
        int ret;
        int port;

        if (args.Length < 1) {
            print_usage();
            return;
        }

        port = Convert.ToInt32(args[0]);

        // Open the interface
        km = KomodoApi.km_open(port);
        if (km <= 0) {
            Console.Write("Unable to open Komodo on port {0}\n", port);
            Console.Write("Error code = {0}\n", km);
            return;
        }

        // Acquire features
        ret = KomodoApi.km_acquire(km, KomodoApi.KM_FEATURE_GPIO_LISTEN  |
                                       KomodoApi.KM_FEATURE_GPIO_CONTROL |
                                       KomodoApi.KM_FEATURE_GPIO_CONFIG);
        Console.Write("Acquired features 0x{0:x}\n", ret);

        _gpio(km);

        // Close and exit
        KomodoApi.km_close(km);
        return;
    }

}
