/*=========================================================================
| (c) 2011  Total Phase, Inc.
|--------------------------------------------------------------------------
| Project : Komodo Examples
| File    : loopback.cs
|--------------------------------------------------------------------------
| Demonstrates how to open ports, acquire features, write and read data.
|--------------------------------------------------------------------------
| Redistribution and use of this file in source and binary forms, with
| or without modification, are permitted.
|
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
| "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
| LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
| FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
| COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
| INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
| BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
| LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
| ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
| POSSIBILITY OF SUCH DAMAGE.
 ========================================================================*/
using System;
using System.IO;
using TotalPhase;

/*=========================================================================
| CLASS
 ========================================================================*/
public class loopback {

    /*=====================================================================
    | USAGE INFORMATION
    =====================================================================*/
    static void print_usage () {
        Console.Write(
"Usage: loopback PORT1 PORT2\n" +
"\n" +
"Example utility showing how to read and write CAN data.\n" +
"CAN channels A and B must be connected with a terminated cable.\n" +
"PORT1 and PORT2 may be the same or different.\n" +
"\n" +
"  PORT1: Komodo port that connects to CAN channel A\n" +
"  PORT2: Komodo port that connects to CAN channel B\n" +
"\n" +
"For product documentation and specifications, see www.totalphase.com.\n");
        Console.Out.Flush();
    }


    /*=====================================================================
    | FUNCTIONS
    =====================================================================*/
    static void print_num_array (byte[] array, int len) {
        int i = 0;

        Console.Write("[ ");
        for (i = 0; i < len; i++)
            Console.Write("{0:x} ", array[i]);
        Console.Write("]");
    }

    static void _loopback (int kma, int kmb) {
        int    i;
        int    ret;
        uint   can_id    = 0xAB;
        byte[] data_out  = { 1, 2, 3, 4, 5, 6, 7, 8 };
        byte   data_len  = (byte)data_out.Length;
        byte[] data_in   = new byte[data_len];
        uint   arb_count = 0;

        KomodoApi.km_can_info_t info = new KomodoApi.km_can_info_t();

        KomodoApi.km_can_packet_t pkt;
        pkt.remote_req  = 0;
        pkt.extend_addr = 0;
        pkt.dlc         = data_len;
        pkt.id          = can_id;

        // Enable CAN A and B
        ret = KomodoApi.km_enable(kma);
        if (ret != (int)km_status_t.KM_OK) {
            Console.Write("Unable to enable Komodo A\n");
            return;
        }

        if (kma != kmb) {
            ret = KomodoApi.km_enable(kmb);
            if (ret != (int)km_status_t.KM_OK) {
                Console.Write("Unable to enable Komodo B\n");
                return;
            }
        }

        // Send data on CAN A
        KomodoApi.km_can_write(kma, km_can_ch_t.KM_CAN_CH_A, 0,
                               pkt, data_len, data_out, ref arb_count);

        Console.Write("Sent data:     ");
        print_num_array(data_out, data_len);

        // Read data on CAN B
        do {
            ret = KomodoApi.km_can_read(kmb, ref info,
                                        ref pkt, data_len, data_in);
        } while(ret == 0);

        Console.Write("\nReceived data: ");
        print_num_array(data_in, ret);

        // Verify received data
        for (i = 0; i < data_len; i++)
            if (data_out[i] != data_in[i])
                break;

        Console.Write("\nVerifying data... {0:s}\n\n",
                      i == data_len ? "PASS" : "FAIL");
    }


    /*=====================================================================
    | MAIN PROGRAM
    =====================================================================*/
    public static void Main (String[] args) {
        int  kma, kmb;
        int  reta, retb;
        int  porta, portb;
        uint timeout = 1000;   // ms
        uint bitrate = 125000; // Hz

        if (args.Length < 2) {
            print_usage();
            return;
        }

        porta = Convert.ToInt32(args[0]);
        portb = Convert.ToInt32(args[1]);

        // Connect to and configure
        kma = KomodoApi.km_open(porta);
        if (kma <= 0) {
            Console.Write("Unable to open Komodo on port {0:d}\n", porta);
            Console.Write("Error code = {0:d}\n", kma);
            return;
        }

        if (porta != portb) {
            kmb = KomodoApi.km_open(portb);
            if (kmb <= 0) {
                Console.Write("Unable to open Komodo on port {0:d}\n", portb);
                Console.Write("Error code = {0:d}\n", kmb);
                return;
            }
        }
        else kmb = kma;

        // Setup CAN channel A for writing and B for listening
        reta = KomodoApi.km_acquire(kma, KomodoApi.KM_FEATURE_CAN_A_CONFIG  |
                                         KomodoApi.KM_FEATURE_CAN_A_CONTROL);
        retb = KomodoApi.km_acquire(kmb, KomodoApi.KM_FEATURE_CAN_B_CONFIG  |
                                         KomodoApi.KM_FEATURE_CAN_B_CONTROL |
                                         KomodoApi.KM_FEATURE_CAN_B_LISTEN);
        Console.Write("Features for CAN A: 0x{0:x}, CAN B: 0x{1:x}\n",
                      reta, retb);

        // Set bitrate
        reta = KomodoApi.km_can_bitrate(kma, km_can_ch_t.KM_CAN_CH_A, bitrate);
        retb = KomodoApi.km_can_bitrate(kmb, km_can_ch_t.KM_CAN_CH_B, bitrate);
        Console.Write("Bitrate for CAN A set to {0:d} kHz, " +
                      "CAN B set to %d{1:d} kHz\n", reta/1000, retb/1000);

        // Set timeout
        KomodoApi.km_timeout(kmb, timeout);
        Console.Write("Timeout for CAN B set to {0:d} ms\n", timeout);

        // Set target power
        KomodoApi.km_can_target_power(kma, km_can_ch_t.KM_CAN_CH_A,
                                      km_power_t.KM_TARGET_POWER_ON);
        KomodoApi.km_can_target_power(kmb, km_can_ch_t.KM_CAN_CH_B,
                                      km_power_t.KM_TARGET_POWER_OFF);
        Console.Write("Enabled target power for CAN A " +
                      "and disabled for CAN B\n\n");

        _loopback(kma, kmb);

        // Close and exit
        KomodoApi.km_close(kma);
        KomodoApi.km_close(kmb);
        return;
    }
}
