/*=========================================================================
| (c) 2011  Total Phase, Inc.
|--------------------------------------------------------------------------
| Project : Komodo Examples
| File    : monitor.cs
|--------------------------------------------------------------------------
| Simple program that monitors CAN bus and GPIO activity
|--------------------------------------------------------------------------
| Redistribution and use of this file in source and binary forms, with
| or without modification, are permitted.
|
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
| "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
| LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
| FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
| COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
| INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
| BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
| LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
| ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
| POSSIBILITY OF SUCH DAMAGE.
 ========================================================================*/
using System;
using System.IO;
using TotalPhase;

/*=========================================================================
| CLASS
 ========================================================================*/
public class monitor {
    const int NUM_GPIOS = 8;


    /*=====================================================================
    | USAGE INFORMATION
     ====================================================================*/
    static void print_usage () {
        Console.Write(
"Usage: monitor TARGET_PWR NUM_EVENTS \n" +
"\n" +
"Example utility for capturing CAN and GPIO activity on CAN A.\n" +
"\n" +
"  TARGET_PWR: 1 turns on target power, 0 does not\n" +
"  NUM_EVENTS: Number of events to process before exiting.  If this is set to\n" +
"              zero, the capture will continue indefinitely\n" +
"\n" +
"For product documentation and specifications, see www.totalphase.com.\n");
        Console.Out.Flush();
}


    /*=====================================================================
    | FUNCTIONS
     ====================================================================*/
    /* This will only work for sample rates that are multiples of 1MHz.
       If any other rates are desired, this code needs to be changed.*/
    static ulong timestamp_to_ns (ulong stamp, int samplerate_khz) {
        return (ulong)(stamp * 1000 / (ulong)(samplerate_khz/1000));
    }

    static void print_num_array (byte[] array, uint len) {
        int i = 0;

        for (i = 0; i < len; i++)
            Console.Write("{0:x} ", array[i]);
    }

    static void print_status (uint status) {
        Console.Write(" ");

        if (status == (int)km_status_t.KM_OK)
            Console.Write("OK ");
        if ((status & KomodoApi.KM_READ_TIMEOUT) != 0)
            Console.Write("TIMEOUT ");
        if ((status & KomodoApi.KM_READ_ERR_OVERFLOW) != 0)
            Console.Write("OVERFLOW ");
        if ((status & KomodoApi.KM_READ_END_OF_CAPTURE) != 0)
            Console.Write("END OF CAPTURE ");
        if ((status & KomodoApi.KM_READ_CAN_ARB_LOST) != 0)
            Console.Write("ARBITRATION LOST ");
        if ((status & KomodoApi.KM_READ_CAN_ERR) != 0)
            Console.Write("ERROR {0:x} ",
                          status & KomodoApi.KM_READ_CAN_ERR_FULL_MASK);
    }

    static void print_events (uint events, uint bitrate) {
        if (events == 0)
            return;
        if ((events & KomodoApi.KM_EVENT_DIGITAL_INPUT) != 0)
            Console.Write("GPIO CHANGE 0x{0:x}; ",
                   events & KomodoApi.KM_EVENT_DIGITAL_INPUT_MASK);
        if ((events & KomodoApi.KM_EVENT_CAN_BUS_STATE_LISTEN_ONLY) != 0)
            Console.Write("BUS STATE LISTEN ONLY; ");
        if ((events & KomodoApi.KM_EVENT_CAN_BUS_STATE_CONTROL) != 0)
            Console.Write("BUS STATE CONTROL; ");
        if ((events & KomodoApi.KM_EVENT_CAN_BUS_STATE_WARNING) != 0)
            Console.Write("BUS STATE WARNING; ");
        if ((events & KomodoApi.KM_EVENT_CAN_BUS_STATE_ACTIVE) != 0)
            Console.Write("BUS STATE ACTIVE; ");
        if ((events & KomodoApi.KM_EVENT_CAN_BUS_STATE_PASSIVE) != 0)
            Console.Write("BUS STATE PASSIVE; ");
        if ((events & KomodoApi.KM_EVENT_CAN_BUS_STATE_OFF) != 0)
            Console.Write("BUS STATE OFF; ");
        if ((events & KomodoApi.KM_EVENT_CAN_BUS_BITRATE) != 0)
            Console.Write("BITRATE {0:d}kHz; ", bitrate/1000);
    }

    static void candump (int km, int max_events) {
        int    ret;
        int    count;
        int    samplerate_khz;
        int    data_len = 8;
        byte[] data     = new byte[data_len];

        KomodoApi.km_can_info_t   info = new KomodoApi.km_can_info_t();
        KomodoApi.km_can_packet_t pkt  = new KomodoApi.km_can_packet_t();

        // Get samplerate
        samplerate_khz = KomodoApi.km_get_samplerate(km)/1000;

        // Enable Komodo
        ret = KomodoApi.km_enable(km);
        if (ret != (int)km_status_t.KM_OK) {
            Console.Write("Unable to enable Komodo\n");
            return;
        }

        // Print description of csv output
        Console.Write("\nindex,time(ns),(status & events),<ID:rtr/data> hex data\n\n");

        // Start monitoring
        for (count = 0; max_events == 0 || count < max_events; count++) {
            ret = KomodoApi.km_can_read(km, ref info, ref pkt, data_len, data);

            Console.Write("{0:d},{1:d},(", count,
                          timestamp_to_ns(info.timestamp, samplerate_khz));

            if (ret < 0)
                Console.Write("error={0:d}", ret);

            print_status(info.status);
            print_events(info.events, info.bitrate_hz);
            Console.Write(")");

            // Continue printing if we didn't see timeout, error or dataless events
            if ( (info.status == (int)km_status_t.KM_OK) && (info.events == 0)) {
                Console.Write(",<{0:x}:{1:s} ", pkt.id,
                              pkt.remote_req == 1 ? "rtr>" : "data>");

                // If packet contained data, print it
                if (pkt.remote_req == 0)
                    print_num_array(data, pkt.dlc);
            }
            Console.Write("\n");
            Console.Out.Flush();
        }
    }


    /*=====================================================================
    | MAIN PROGRAM
     ====================================================================*/
    public static void Main (String[] args) {
        int        km;
        int        i;
        int        ret;
        km_power_t power;
        int        max_events;
        int        port    = 0;      // Use port 0
        ushort     timeout = 1000;   // ms
        uint       bitrate = 125000; // Hz

        if (args.Length < 2) {
            print_usage();
            return;
        }

        power  = Convert.ToInt32(args[0]) == 1 ?
                 km_power_t.KM_TARGET_POWER_ON :
                 km_power_t.KM_TARGET_POWER_OFF;
        max_events = Convert.ToInt32(args[1]);

        // Open the interface
        km = KomodoApi.km_open(port);
        if (km <= 0) {
            Console.Write("Unable to open Komodo on port {0:d}\n", port);
            Console.Write("Error code = {0:d}\n", km);
            return;
        }

        /* Acquire features.  Acquiring KM_FEATURE_CAN_A_CONTROL causes the
           Komodo interface to ACK all packets transmitted on the bus.  Remove
           this feature to prevent the device from transmitting anything on the
           bus.*/
        ret = KomodoApi.km_acquire(km, KomodoApi.KM_FEATURE_CAN_A_CONFIG  |
                                       KomodoApi.KM_FEATURE_CAN_A_LISTEN  |
                                       KomodoApi.KM_FEATURE_CAN_A_CONTROL |
                                       KomodoApi.KM_FEATURE_GPIO_CONFIG   |
                                       KomodoApi.KM_FEATURE_GPIO_LISTEN);
        Console.Write("Acquired features 0x{0:x}\n", ret);

        // Set bitrate
        ret = KomodoApi.km_can_bitrate(km, km_can_ch_t.KM_CAN_CH_A, bitrate);
        Console.Write("Bitrate set to {0:d} kHz\n", ret/1000);

        // Set timeout
        KomodoApi.km_timeout(km, timeout);
        Console.Write("Timeout set to {0:d} ms\n", timeout);

        // Set target power
        KomodoApi.km_can_target_power(km, km_can_ch_t.KM_CAN_CH_A, power);
        Console.Write("Target power {0:d}\n",
               power == km_power_t.KM_TARGET_POWER_ON ? "ON" : "OFF");

        // Configure all GPIO pins as inputs
        for (i = 0; i < NUM_GPIOS; i++)
            KomodoApi.km_gpio_config_in(km, (byte)i,
                              (byte)km_pin_bias_t.KM_PIN_BIAS_PULLUP,
                              (byte)km_pin_trigger_t.KM_PIN_TRIGGER_BOTH_EDGES);
        Console.Write("All pins set as inputs\n");

        candump(km, max_events);

        // Close and exit
        KomodoApi.km_close(km);
        return;
    }
}
