/*=========================================================================
| (c) 2011  Total Phase, Inc.
|--------------------------------------------------------------------------
| Project : Komodo Examples
| File    : request.cs
|--------------------------------------------------------------------------
| Request data from specified CAN ID and print received data.
|--------------------------------------------------------------------------
| Redistribution and use of this file in source and binary forms, with
| or without modification, are permitted.
|
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
| "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
| LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
| FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
| COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
| INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
| BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
| LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
| ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
| POSSIBILITY OF SUCH DAMAGE.
 ========================================================================*/
using System;
using System.IO;
using TotalPhase;

/*=========================================================================
| CLASS
 ========================================================================*/
public class request_data {

    /*=====================================================================
    | USAGE INFORMATION
    =====================================================================*/
    static void print_usage () {
        Console.Write(
"Usage: request PORT TARGET_PWR CAN_ID \n" +
"\n" +
"Example utility which requests data from provided CAN ID on CAN A.\n" +
"\n" +
"  PORT: Komodo port\n" +
"  TARGET_PWR: 1 turns on target power, 0 does not" +
"  CAN_ID: ID to request CAN packet from\n" +
"\n" +
"For product documentation and specifications, see www.totalphase.com.\n");
        Console.Out.Flush();
    }


    /*=====================================================================
    | FUNCTIONS
    =====================================================================*/
    static void print_num_array (byte[] array, int len) {
        int i = 0;

        Console.Write("[ ");
        for (i = 0; i < len; i++)
            Console.Write("{0:x} ", array[i]);
        Console.Write("]");
    }

    static void request (int km, uint can_id, km_can_ch_t ch) {
        int    ret;
        uint   arb_count = 0;
        byte   data_len  = 8;
        byte[] data      = new byte[data_len];

        KomodoApi.km_can_info_t info = new KomodoApi.km_can_info_t();

        KomodoApi.km_can_packet_t pkt;
        pkt.remote_req  = 1;
        pkt.extend_addr = 0;
        pkt.dlc         = data_len;
        pkt.id          = can_id;

        // Enable Komodo
        ret = KomodoApi.km_enable(km);
        if (ret != (int)km_status_t.KM_OK) {
            Console.Write("Unable to enable Komodo\n");
            return;
        }

        Console.Write("Request data from CAN ID {0:d} (0x{0:x})\n", can_id);

        // Send a remote frame with no data.  5th arg is 0 and
        // data array is not important.
        KomodoApi.km_can_write(km, ch, 0, pkt, 0, data, ref arb_count);

        // Look for packet with desired CAN ID. Store in data[].
        do {
            ret = KomodoApi.km_can_read(km, ref info, ref pkt, data_len, data);
            if ((info.status & KomodoApi.KM_READ_TIMEOUT) != 0) {
                Console.Write("Timeout: no data!\n\n");
                return;
            }
        } while (pkt.id != can_id);

        Console.Write("Received {0:d} byte(s): ", ret);
        Console.Out.Flush();

        print_num_array(data, ret);
        Console.Write("\n\n");
    }


    /*=====================================================================
    | MAIN PROGRAM
    =====================================================================*/
    public static void Main (String[] args) {
        int        km;
        int        ret;
        int        port;
        km_power_t power;
        uint       can_id;
        uint       bitrate  = 125000; // kHz
        uint       timeout  = 3000;   // ms

        if (args.Length < 3) {
            print_usage();
            return;
        }

        port   = Convert.ToInt32(args[0]);
        power  = Convert.ToInt32(args[1]) == 1 ?
                 km_power_t.KM_TARGET_POWER_ON :
                 km_power_t.KM_TARGET_POWER_OFF;
        can_id = args[2].IndexOf("0x") == 0 ?
                 Convert.ToUInt32(args[2], 16) :
                 Convert.ToUInt32(args[2]);

        // Open the interface
        km = KomodoApi.km_open(port);
        if (km <= 0) {
            Console.Write("Unable to open Komodo on port {0:d}\n", port);
            Console.Write("Error code = {0:d}\n", km);
            return;
        }

        // Acquire features to control and listen CAN A
        ret = KomodoApi.km_acquire(km, KomodoApi.KM_FEATURE_CAN_A_CONFIG  |
                                       KomodoApi.KM_FEATURE_CAN_A_CONTROL |
                                       KomodoApi.KM_FEATURE_CAN_A_LISTEN);
        Console.Write("Acquired features: 0x{0:x}\n", ret);

        // Set bitrate
        ret = KomodoApi.km_can_bitrate(km, km_can_ch_t.KM_CAN_CH_A, bitrate);
        Console.Write("Bitrate set to {0:d} kHz\n", ret/1000);

        // Set timeout
        KomodoApi.km_timeout(km, timeout);
        Console.Write("Timeout set to {0:d} ms\n", timeout);

        // Set target power
        KomodoApi.km_can_target_power(km, km_can_ch_t.KM_CAN_CH_A, power);
        Console.Write("Target power {0:s}\n\n",
                      power == km_power_t.KM_TARGET_POWER_ON ? "ON" : "OFF");

        request(km, can_id, km_can_ch_t.KM_CAN_CH_A);

        // Close and exit
        KomodoApi.km_close(km);
        return;
    }
}
