Attribute VB_Name = "KOMODO"
'==========================================================================
' Komodo Interface Library
'--------------------------------------------------------------------------
' Copyright (c) 2010-2024 Total Phase, Inc.
' All rights reserved.
' www.totalphase.com
'
' Redistribution and use of this file in source and binary forms, with
' or without modification, are permitted provided that the following
' conditions are met:
'
' - Redistributions of source code must retain the above copyright
'   notice, this list of conditions, and the following disclaimer.
'
' - Redistributions in binary form must reproduce the above copyright
'   notice, this list of conditions, and the following disclaimer in the
'   documentation or other materials provided with the distribution.
'
' - This file must only be used to interface with Total Phase products.
'   The names of Total Phase and its contributors must not be used to
'   endorse or promote products derived from this software.
'
' THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
' "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING BUT NOT
' LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
' FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.  IN NO EVENT WILL THE
' COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
' INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING
' BUT NOT LIMITED TO PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
' LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
' CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
' LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
' ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
' POSSIBILITY OF SUCH DAMAGE.
'--------------------------------------------------------------------------
' To access Total Phase Komodo devices through the API:
'
' 1) Use one of the following shared objects:
'      komodo.so       --  Linux or macOS shared object
'      komodo.dll      --  Windows dynamic link library
'
' 2) Along with one of the following language modules:
'      komodo.c/h      --  C/C++ API header file and interface module
'      komodo_py.py    --  Python API
'      komodo.cs       --  C# .NET source
'      komodo_net.dll  --  Compiled .NET binding
'      komodo.bas      --  Visual Basic 6 API
'==========================================================================

' Data types:
'    Handles are Long
'
'    Fixed bit-width types
'      8-bit => Byte
'     16-bit => Integer
'     32-bit => Long
'     64-bit => LongLong

Public Type LongLong
    LoValue As Long
    HiValue As Long
End Type

Public Enum km_status_t
    ' General codes (0 to -99)
    KM_OK                      =    0
    KM_UNABLE_TO_LOAD_LIBRARY  =   -1
    KM_UNABLE_TO_LOAD_DRIVER   =   -2
    KM_UNABLE_TO_LOAD_FUNCTION =   -3
    KM_INCOMPATIBLE_LIBRARY    =   -4
    KM_INCOMPATIBLE_DEVICE     =   -5
    KM_COMMUNICATION_ERROR     =   -6
    KM_UNABLE_TO_OPEN          =   -7
    KM_UNABLE_TO_CLOSE         =   -8
    KM_INVALID_HANDLE          =   -9
    KM_CONFIG_ERROR            =  -10
    KM_PARAM_ERROR             =  -11
    KM_FUNCTION_NOT_AVAILABLE  =  -12
    KM_FEATURE_NOT_ACQUIRED    =  -13
    KM_NOT_DISABLED            =  -14
    KM_NOT_ENABLED             =  -15

    ' CAN codes (-100 to -199)
    KM_CAN_READ_EMPTY          = -101
    KM_CAN_SEND_TIMEOUT        = -102
    KM_CAN_SEND_FAIL           = -103
    KM_CAN_ASYNC_EMPTY         = -104
    KM_CAN_ASYNC_MAX_REACHED   = -105
    KM_CAN_ASYNC_PENDING       = -106
    KM_CAN_ASYNC_TIMEOUT       = -107
    KM_CAN_AUTO_BITRATE_FAIL   = -108
End Enum

Public Type KomodoVersion
    ' Software, firmware, and hardware versions.
    software         as Integer
    firmware         as Integer
    hardware         as Integer

    ' Firmware revisions that are compatible with this software version.
    ' The top 16 bits gives the maximum accepted fw revision.
    ' The lower 16 bits gives the minimum accepted fw revision.
    fw_revs_for_sw   as Long

    ' Hardware revisions that are compatible with this software version.
    ' The top 16 bits gives the maximum accepted hw revision.
    ' The lower 16 bits gives the minimum accepted hw revision.
    hw_revs_for_sw   as Long

    ' Software requires that the API interface must be >= this version.
    api_req_by_sw    as Integer
End Type

Public Const KM_FEATURE_GPIO_LISTEN = &H00000001
Public Const KM_FEATURE_GPIO_CONTROL = &H00000002
Public Const KM_FEATURE_GPIO_CONFIG = &H00000004
Public Const KM_FEATURE_CAN_A_LISTEN = &H00000008
Public Const KM_FEATURE_CAN_A_CONTROL = &H00000010
Public Const KM_FEATURE_CAN_A_CONFIG = &H00000020
Public Const KM_FEATURE_CAN_B_LISTEN = &H00000040
Public Const KM_FEATURE_CAN_B_CONTROL = &H00000080
Public Const KM_FEATURE_CAN_B_CONFIG = &H00000100

Public Const KM_PORT_NOT_FREE = &H8000
Public Const KM_PORT_NUM_MASK = &H00ff

Public Type KomodoExt
    ' Version matrix
    version    as KomodoVersion

    ' Features of this device.
    features   as Long
End Type

Public Const KM_TIMEOUT_IMMEDIATE = 0
Public Const KM_TIMEOUT_INFINITE = -1

Public Enum km_can_ch_t
    KM_CAN_CH_A = 0
    KM_CAN_CH_B = 1
End Enum

Public Const KM_CAN_BUS_STATE_LISTEN_ONLY = &H00000001
Public Const KM_CAN_BUS_STATE_CONTROL = &H00000002
Public Const KM_CAN_BUS_STATE_WARNING = &H00000004
Public Const KM_CAN_BUS_STATE_ACTIVE = &H00000008
Public Const KM_CAN_BUS_STATE_PASSIVE = &H00000010
Public Const KM_CAN_BUS_STATE_OFF = &H00000020

Public Const KM_CAN_CONFIG_NONE = &H00000000
Public Const KM_CAN_CONFIG_LISTEN_SELF = &H00000001

Public Const KM_KHZ = 1000
Public Const KM_MHZ = 1000000

Public Enum km_power_t
    KM_TARGET_POWER_QUERY = &H02
    KM_TARGET_POWER_OFF   = &H00
    KM_TARGET_POWER_ON    = &H01
End Enum

Public Const KM_READ_TIMEOUT = &H80000000
Public Const KM_READ_ERR_OVERFLOW = &H40000000
Public Const KM_READ_END_OF_CAPTURE = &H20000000
Public Const KM_READ_CAN_ERR = &H00000100
Public Const KM_READ_CAN_ERR_FULL_MASK = &H000000ff
Public Const KM_READ_CAN_ERR_POS_MASK = &H0000001f
Public Const KM_READ_CAN_ERR_POS_SOF = &H00000003
Public Const KM_READ_CAN_ERR_POS_ID28_21 = &H00000002
Public Const KM_READ_CAN_ERR_POS_ID20_18 = &H00000006
Public Const KM_READ_CAN_ERR_POS_SRTR = &H00000004
Public Const KM_READ_CAN_ERR_POS_IDE = &H00000005
Public Const KM_READ_CAN_ERR_POS_ID17_13 = &H00000007
Public Const KM_READ_CAN_ERR_POS_ID12_5 = &H0000000f
Public Const KM_READ_CAN_ERR_POS_ID4_0 = &H0000000e
Public Const KM_READ_CAN_ERR_POS_RTR = &H0000000c
Public Const KM_READ_CAN_ERR_POS_RSVD_1 = &H0000000d
Public Const KM_READ_CAN_ERR_POS_RSVD_0 = &H00000009
Public Const KM_READ_CAN_ERR_POS_DLC = &H0000000b
Public Const KM_READ_CAN_ERR_POS_DF = &H0000000a
Public Const KM_READ_CAN_ERR_POS_CRC_SEQ = &H00000008
Public Const KM_READ_CAN_ERR_POS_CRC_DEL = &H00000018
Public Const KM_READ_CAN_ERR_POS_ACK_SLOT = &H00000019
Public Const KM_READ_CAN_ERR_POS_ACK_DEL = &H0000001b
Public Const KM_READ_CAN_ERR_POS_EOF = &H0000001a
Public Const KM_READ_CAN_ERR_POS_INTRMSN = &H00000012
Public Const KM_READ_CAN_ERR_POS_AEF = &H00000011
Public Const KM_READ_CAN_ERR_POS_PEF = &H00000016
Public Const KM_READ_CAN_ERR_POS_TDB = &H00000013
Public Const KM_READ_CAN_ERR_POS_ERR_DEL = &H00000017
Public Const KM_READ_CAN_ERR_POS_OVRFLG = &H0000001c
Public Const KM_READ_CAN_ERR_DIR_MASK = &H00000020
Public Const KM_READ_CAN_ERR_DIR_TX = &H00000000
Public Const KM_READ_CAN_ERR_DIR_RX = &H00000020
Public Const KM_READ_CAN_ERR_TYPE_MASK = &H000000c0
Public Const KM_READ_CAN_ERR_TYPE_BIT = &H00000000
Public Const KM_READ_CAN_ERR_TYPE_FORM = &H00000040
Public Const KM_READ_CAN_ERR_TYPE_STUFF = &H00000080
Public Const KM_READ_CAN_ERR_TYPE_OTHER = &H000000c0
Public Const KM_READ_CAN_ARB_LOST = &H00000200
Public Const KM_READ_CAN_ARB_LOST_POS_MASK = &H000000ff

Public Const KM_GPIO_PIN_1_CONFIG = &H00
Public Const KM_GPIO_PIN_2_CONFIG = &H01
Public Const KM_GPIO_PIN_3_CONFIG = &H02
Public Const KM_GPIO_PIN_4_CONFIG = &H03
Public Const KM_GPIO_PIN_5_CONFIG = &H04
Public Const KM_GPIO_PIN_6_CONFIG = &H05
Public Const KM_GPIO_PIN_7_CONFIG = &H06
Public Const KM_GPIO_PIN_8_CONFIG = &H07

Public Const KM_GPIO_PIN_1_MASK = &H01
Public Const KM_GPIO_PIN_2_MASK = &H02
Public Const KM_GPIO_PIN_3_MASK = &H04
Public Const KM_GPIO_PIN_4_MASK = &H08
Public Const KM_GPIO_PIN_5_MASK = &H10
Public Const KM_GPIO_PIN_6_MASK = &H20
Public Const KM_GPIO_PIN_7_MASK = &H40
Public Const KM_GPIO_PIN_8_MASK = &H80

Public Const KM_EVENT_DIGITAL_INPUT = &H00000100
Public Const KM_EVENT_DIGITAL_INPUT_MASK = &H000000ff
Public Const KM_EVENT_DIGITAL_INPUT_1 = &H00000001
Public Const KM_EVENT_DIGITAL_INPUT_2 = &H00000002
Public Const KM_EVENT_DIGITAL_INPUT_3 = &H00000004
Public Const KM_EVENT_DIGITAL_INPUT_4 = &H00000008
Public Const KM_EVENT_DIGITAL_INPUT_5 = &H00000010
Public Const KM_EVENT_DIGITAL_INPUT_6 = &H00000020
Public Const KM_EVENT_DIGITAL_INPUT_7 = &H00000040
Public Const KM_EVENT_DIGITAL_INPUT_8 = &H00000080
Public Const KM_EVENT_CAN_BUS_STATE_LISTEN_ONLY = &H00001000
Public Const KM_EVENT_CAN_BUS_STATE_CONTROL = &H00002000
Public Const KM_EVENT_CAN_BUS_STATE_WARNING = &H00004000
Public Const KM_EVENT_CAN_BUS_STATE_ACTIVE = &H00008000
Public Const KM_EVENT_CAN_BUS_STATE_PASSIVE = &H00010000
Public Const KM_EVENT_CAN_BUS_STATE_OFF = &H00020000
Public Const KM_EVENT_CAN_BUS_BITRATE = &H00040000

Public Type km_can_info_t
    timestamp        as LongLong
    status           as Long
    events           as Long
    channel          as Long
    bitrate_hz       as Long
    host_gen         as Byte
    rx_error_count   as Byte
    tx_error_count   as Byte
    overflow_count   as Long
End Type

Public Type km_can_packet_t
    remote_req    as Byte
    extend_addr   as Byte
    dlc           as Byte
    id            as Long
End Type

Public Const KM_CAN_ONE_SHOT = &H01

Public Enum km_pin_bias_t
    KM_PIN_BIAS_TRISTATE = &H00
    KM_PIN_BIAS_PULLUP   = &H01
    KM_PIN_BIAS_PULLDOWN = &H02
End Enum

Public Enum km_pin_trigger_t
    KM_PIN_TRIGGER_NONE         = &H00
    KM_PIN_TRIGGER_RISING_EDGE  = &H01
    KM_PIN_TRIGGER_FALLING_EDGE = &H02
    KM_PIN_TRIGGER_BOTH_EDGES   = &H03
End Enum

Public Enum km_pin_drive_t
    KM_PIN_DRIVE_NORMAL            = &H00
    KM_PIN_DRIVE_INVERTED          = &H01
    KM_PIN_DRIVE_OPEN_DRAIN        = &H02
    KM_PIN_DRIVE_OPEN_DRAIN_PULLUP = &H03
End Enum

Public Enum km_pin_source_t
    KM_PIN_SRC_SOFTWARE_CTL       = &H00
    KM_PIN_SRC_ALL_ERR_CAN_A      = &H11
    KM_PIN_SRC_BIT_ERR_CAN_A      = &H12
    KM_PIN_SRC_FORM_ERR_CAN_A     = &H13
    KM_PIN_SRC_STUFF_ERR_CAN_A    = &H14
    KM_PIN_SRC_OTHER_ERR_CAN_A    = &H15
    KM_PIN_SRC_ALL_ERR_CAN_B      = &H21
    KM_PIN_SRC_BIT_ERR_CAN_B      = &H22
    KM_PIN_SRC_FORM_ERR_CAN_B     = &H23
    KM_PIN_SRC_STUFF_ERR_CAN_B    = &H24
    KM_PIN_SRC_OTHER_ERR_CAN_B    = &H25
    KM_PIN_SRC_ALL_ERR_CAN_BOTH   = &H31
    KM_PIN_SRC_BIT_ERR_CAN_BOTH   = &H32
    KM_PIN_SRC_FORM_ERR_CAN_BOTH  = &H33
    KM_PIN_SRC_STUFF_ERR_CAN_BOTH = &H34
    KM_PIN_SRC_OTHER_ERR_CAN_BOTH = &H35
End Enum

Private Declare Function vb6_km_find_devices Lib "komodo.dll" (ByVal num_ports As Long, ByRef ports As Integer) as Long
Private Declare Function vb6_km_find_devices_ext Lib "komodo.dll" (ByVal num_ports As Long, ByRef ports As Integer, ByVal num_ids As Long, ByRef unique_ids As Long) as Long
Private Declare Function vb6_km_open Lib "komodo.dll" (ByVal port_number As Long) as Long
Private Declare Function vb6_km_open_ext Lib "komodo.dll" (ByVal port_number As Long, ByRef km_ext As KomodoExt) as Long
Private Declare Function vb6_km_close Lib "komodo.dll" (ByVal komodo As Long) as Long
Private Declare Function vb6_km_port Lib "komodo.dll" (ByVal komodo As Long) as Long
Private Declare Function vb6_km_features Lib "komodo.dll" (ByVal komodo As Long) as Long
Private Declare Function vb6_km_unique_id Lib "komodo.dll" (ByVal komodo As Long) as Long
Private Declare Function vb6_km_status_string Lib "komodo.dll" (ByVal status As Long) as String
Private Declare Function vb6_km_version Lib "komodo.dll" (ByVal komodo As Long, ByRef version As KomodoVersion) as Long
Private Declare Function vb6_km_sleep_ms Lib "komodo.dll" (ByVal milliseconds As Long) as Long
Private Declare Function vb6_km_acquire Lib "komodo.dll" (ByVal komodo As Long, ByVal features As Long) as Long
Private Declare Function vb6_km_release Lib "komodo.dll" (ByVal komodo As Long, ByVal features As Long) as Long
Private Declare Function vb6_km_timeout Lib "komodo.dll" (ByVal komodo As Long, ByVal timeout_ms As Long) as Long
Private Declare Function vb6_km_can_query_bus_state Lib "komodo.dll" (ByVal komodo As Long, ByVal channel As Long, ByRef bus_state As Byte, ByRef rx_error As Byte, ByRef tx_error As Byte) as Long
Private Declare Function vb6_km_latency Lib "komodo.dll" (ByVal komodo As Long, ByVal latency_ms As Long) as Long
Private Declare Function vb6_km_can_configure Lib "komodo.dll" (ByVal komodo As Long, ByVal config As Long) as Long
Private Declare Function vb6_km_can_bus_timeout Lib "komodo.dll" (ByVal komodo As Long, ByVal channel As Long, ByVal timeout_ms As Integer) as Long
Private Declare Function vb6_km_can_bitrate Lib "komodo.dll" (ByVal komodo As Long, ByVal channel As Long, ByVal bitrate_hz As Long) as Long
Private Declare Function vb6_km_can_auto_bitrate Lib "komodo.dll" (ByVal komodo As Long, ByVal channel As Long) as Long
Private Declare Function vb6_km_can_auto_bitrate_ext Lib "komodo.dll" (ByVal komodo As Long, ByVal channel As Long, ByVal num_bitrates_hz As Long, ByRef bitrates_hz As Long) as Long
Private Declare Function vb6_km_get_samplerate Lib "komodo.dll" (ByVal komodo As Long) as Long
Private Declare Function vb6_km_can_target_power Lib "komodo.dll" (ByVal komodo As Long, ByVal channel As Long, ByVal power As Long) as Long
Private Declare Function vb6_km_enable Lib "komodo.dll" (ByVal komodo As Long) as Long
Private Declare Function vb6_km_disable Lib "komodo.dll" (ByVal komodo As Long) as Long
Private Declare Function vb6_km_can_read Lib "komodo.dll" (ByVal komodo As Long, ByRef info As km_can_info_t, ByRef pkt As km_can_packet_t, ByVal num_bytes As Long, ByRef data As Byte) as Long
Private Declare Function vb6_km_can_async_submit Lib "komodo.dll" (ByVal komodo As Long, ByVal channel As Long, ByVal flags As Byte, ByRef pkt As km_can_packet_t, ByVal num_bytes As Long, ByRef data As Byte) as Long
Private Declare Function vb6_km_can_async_collect Lib "komodo.dll" (ByVal komodo As Long, ByVal timeout_ms As Long, ByRef arbitration_count As Long) as Long
Private Declare Function vb6_km_can_write Lib "komodo.dll" (ByVal komodo As Long, ByVal channel As Long, ByVal flags As Byte, ByRef pkt As km_can_packet_t, ByVal num_bytes As Long, ByRef data As Byte, ByRef arbitration_count As Long) as Long
Private Declare Function vb6_km_gpio_config_in Lib "komodo.dll" (ByVal komodo As Long, ByVal pin_number As Byte, ByVal bias As Byte, ByVal trigger As Byte) as Long
Private Declare Function vb6_km_gpio_config_out Lib "komodo.dll" (ByVal komodo As Long, ByVal pin_number As Byte, ByVal drive As Byte, ByVal source As Byte) as Long
Private Declare Function vb6_km_gpio_set Lib "komodo.dll" (ByVal komodo As Long, ByVal value As Byte, ByVal mask As Byte) as Long
Private Declare Function vb6_km_gpio_get Lib "komodo.dll" (ByVal komodo As Long) as Long


'==========================================================================
' VERSION
'==========================================================================
Public Const KM_API_VERSION    = &H0200   ' v2.00
Public Const KM_REQ_SW_VERSION = &H010a   ' v1.10

Private Declare Function km_std_version Lib "komodo.dll" () as Long
Dim KM_VERSION_OK as Boolean

Private Function check_version () as Boolean
    If KM_VERSION_OK Then
        check_version = True
    Else
        Dim sw_version As Long
        Dim req_api_version As Long
        sw_version = km_std_version() And &Hffff
        req_api_version = (km_std_version() / 65536) And &Hffff
        KM_VERSION_OK = ((sw_version >= KM_REQ_SW_VERSION) And (KM_API_VERSION >= req_api_version))
        check_version = KM_VERSION_OK
    End If
End Function

'==========================================================================
' HELPER FUNCTIONS
'==========================================================================
Public Function tp_min (ByVal x As Variant, ByVal y As Variant) as Variant
    If x > y Then
        tp_min = y
    Else
        tp_min = x
    End If
End Function


'==========================================================================
' STATUS CODES
'==========================================================================
' All API functions return an integer which is the result of the
' transaction, or a status code if negative.  The status codes are
' defined as follows:

'==========================================================================
' GENERAL TYPE DEFINITIONS
'==========================================================================
' Komodo handle type definition
' typedef Komodo => Long

' Komodo version matrix.
'
' This matrix describes the various version dependencies
' of Komodo components.  It can be used to determine
' which component caused an incompatibility error.
'
' All version numbers are of the format:
'   (major << 8) | minor
'
' ex. v1.20 would be encoded as:  0x0114
' Komodo feature set
'
' This bitmask field describes the features available on this device.
'
' When returned by km_features() or km_open_ext(), it refers to the
' potential features of the device.
' When used as a parameter by km_enable() or km_disable(), it refers
' to the features that the user wants to use.
' And when returned by km_disable(), it refers to the features currently
' in use by the user.

'==========================================================================
' GENERAL API
'==========================================================================
' Get a list of ports to which Komodo devices are attached.
'
' nelem   = maximum number of elements to return
' devices = array into which the port numbers are returned
'
' Each element of the array is written with the port number.
' Devices that are in-use are ORed with KM_PORT_NOT_FREE (0x8000).
'
' ex.  devices are attached to ports 0, 1, 2
'      ports 0 and 2 are available, and port 1 is in-use.
'      array => 0x0000, 0x8001, 0x0002
'
' If the array is NULL, it is not filled with any values.
' If there are more devices than the array size, only the
' first nmemb port numbers will be written into the array.
'
' Returns the number of devices found, regardless of the
' array size.

Public Function km_find_devices (ByVal num_ports As Long, ByRef ports() As Integer) as Long
    If check_version() Then
        Dim ports_num_ports As Long
        ports_num_ports = tp_min(num_ports, UBound(ports) - LBound(ports) + 1)
        km_find_devices = vb6_km_find_devices(ports_num_ports, ports(0))
    Else
        km_find_devices = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Get a list of ports to which Komodo devices are attached.
'
' This function is the same as km_find_devices() except that
' it returns the unique IDs of each Komodo device.  The IDs
' are guaranteed to be non-zero if valid.
'
' The IDs are the unsigned integer representation of the 10-digit
' serial numbers.

Public Function km_find_devices_ext (ByVal num_ports As Long, ByRef ports() As Integer, ByVal num_ids As Long, ByRef unique_ids() As Long) as Long
    If check_version() Then
        Dim ports_num_ports As Long
        ports_num_ports = tp_min(num_ports, UBound(ports) - LBound(ports) + 1)
        Dim unique_ids_num_ids As Long
        unique_ids_num_ids = tp_min(num_ids, UBound(unique_ids) - LBound(unique_ids) + 1)
        km_find_devices_ext = vb6_km_find_devices_ext(ports_num_ports, ports(0), unique_ids_num_ids, unique_ids(0))
    Else
        km_find_devices_ext = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Open the Komodo port.
'
' The port number is a zero-indexed integer.
'
' The port number is the same as that obtained from the
' km_find_devices() function above.
'
' Returns an Komodo handle, which is guaranteed to be
' greater than zero if it is valid.
'
' This function is recommended for use in simple applications
' where extended information is not required.  For more complex
' applications, the use of km_open_ext() is recommended.

Public Function km_open (ByVal port_number As Long) as Long
    If check_version() Then
        km_open = vb6_km_open(port_number)
    Else
        km_open = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Open the Komodo port, returning extended information
' in the supplied structure.  Behavior is otherwise identical
' to km_open() above.  If 0 is passed as the pointer to the
' structure, this function is exactly equivalent to km_open().
'
' The structure is zeroed before the open is attempted.
' It is filled with whatever information is available.
'
' For example, if the firmware version is not filled, then
' the device could not be queried for its version number.
'
' The feature list is a bitmap of Komodo resources, with the same
' mapping as obtained from the km_features() function below.
' Details on the bitmask are found above.
'
' This function is recommended for use in complex applications
' where extended information is required.  For more simple
' applications, the use of km_open() is recommended.

Public Function km_open_ext (ByVal port_number As Long, ByRef km_ext As KomodoExt) as Long
    If check_version() Then
        km_open_ext = vb6_km_open_ext(port_number, km_ext)
    Else
        km_open_ext = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Close the Komodo port.

Public Function km_close (ByVal komodo As Long) as Long
    If check_version() Then
        km_close = vb6_km_close(komodo)
    Else
        km_close = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Return the port for this Komodo handle.
'
' The port number is a zero-indexed integer identical to those
' returned by km_find_devices() above.  This includes the count of
' interfaces in use in the upper byte.

Public Function km_port (ByVal komodo As Long) as Long
    If check_version() Then
        km_port = vb6_km_port(komodo)
    Else
        km_port = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Return the device features as a bit-mask of values, or an error code
' if the handle is not valid.  Details on the bitmask are found above.

Public Function km_features (ByVal komodo As Long) as Long
    If check_version() Then
        km_features = vb6_km_features(komodo)
    Else
        km_features = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Return the unique ID for this Komodo adapter.
' IDs are guaranteed to be non-zero if valid.
' The ID is the unsigned integer representation of the
' 10-digit serial number.

Public Function km_unique_id (ByVal komodo As Long) as Long
    If check_version() Then
        km_unique_id = vb6_km_unique_id(komodo)
    Else
        km_unique_id = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Return the status string for the given status code.
' If the code is not valid or the library function cannot
' be loaded, return a NULL string.

Public Function km_status_string (ByVal status As Long) as String
    If check_version() Then
        km_status_string = vb6_km_status_string(status)
    Else
        km_status_string = vbNullString
    End If
End Function


' Return the version matrix for the device attached to the
' given handle.  If the handle is 0 or invalid, only the
' software and required api versions are set.

Public Function km_version (ByVal komodo As Long, ByRef version As KomodoVersion) as Long
    If check_version() Then
        km_version = vb6_km_version(komodo, version)
    Else
        km_version = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Sleep for the specified number of milliseconds.
' Accuracy depends on the operating system scheduler.
' Returns the number of milliseconds slept.

Public Function km_sleep_ms (ByVal milliseconds As Long) as Long
    If check_version() Then
        km_sleep_ms = vb6_km_sleep_ms(milliseconds)
    Else
        km_sleep_ms = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Acquire device features.
' Returns the features that are currently acquired.

Public Function km_acquire (ByVal komodo As Long, ByVal features As Long) as Long
    If check_version() Then
        km_acquire = vb6_km_acquire(komodo, features)
    Else
        km_acquire = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Release device features.
' Returns the features that are still acquired.

Public Function km_release (ByVal komodo As Long, ByVal features As Long) as Long
    If check_version() Then
        km_release = vb6_km_release(komodo, features)
    Else
        km_release = KM_INCOMPATIBLE_LIBRARY
    End If
End Function



'==========================================================================
' CAN API
'==========================================================================
' These special timeout constants can be used with the functions
' km_timeout and km_can_async_collect.
' Set the timeout of the km_can_read function to the specified
' number of milliseconds.

Public Function km_timeout (ByVal komodo As Long, ByVal timeout_ms As Long) as Long
    If check_version() Then
        km_timeout = vb6_km_timeout(komodo, timeout_ms)
    Else
        km_timeout = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Standard enumeration for the CAN channels available on the Komodo.
' CAN Bus state constants.
' Retreive the current bus state of the supplied CAN channel

Public Function km_can_query_bus_state (ByVal komodo As Long, ByVal channel As Long, ByRef bus_state As Byte, ByRef rx_error As Byte, ByRef tx_error As Byte) as Long
    If check_version() Then
        km_can_query_bus_state = vb6_km_can_query_bus_state(komodo, channel, bus_state, rx_error, tx_error)
    Else
        km_can_query_bus_state = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Set the capture latency to the specified number of milliseconds.
' This number determines the minimum time that a read call will
' block if there is no available data.  Lower times result in
' faster turnaround at the expense of reduced buffering.  Setting
' this parameter too low can cause packets to be dropped.

Public Function km_latency (ByVal komodo As Long, ByVal latency_ms As Long) as Long
    If check_version() Then
        km_latency = vb6_km_latency(komodo, latency_ms)
    Else
        km_latency = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Config mask for km_can_configure

Public Function km_can_configure (ByVal komodo As Long, ByVal config As Long) as Long
    If check_version() Then
        km_can_configure = vb6_km_can_configure(komodo, config)
    Else
        km_can_configure = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Set the bus timeout.  If a zero is passed as the timeout,
' the timeout is unchanged and the current timeout is returned.

Public Function km_can_bus_timeout (ByVal komodo As Long, ByVal channel As Long, ByVal timeout_ms As Integer) as Long
    If check_version() Then
        km_can_bus_timeout = vb6_km_can_bus_timeout(komodo, channel, timeout_ms)
    Else
        km_can_bus_timeout = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Set the CAN bit rate in hertz on the given channel.  If a zero is
' passed as the bitrate, the bitrate is unchanged.  In all cases, the
' call will return the bitrate that will be in effect.

Public Function km_can_bitrate (ByVal komodo As Long, ByVal channel As Long, ByVal bitrate_hz As Long) as Long
    If check_version() Then
        km_can_bitrate = vb6_km_can_bitrate(komodo, channel, bitrate_hz)
    Else
        km_can_bitrate = KM_INCOMPATIBLE_LIBRARY
    End If
End Function



Public Function km_can_auto_bitrate (ByVal komodo As Long, ByVal channel As Long) as Long
    If check_version() Then
        km_can_auto_bitrate = vb6_km_can_auto_bitrate(komodo, channel)
    Else
        km_can_auto_bitrate = KM_INCOMPATIBLE_LIBRARY
    End If
End Function



Public Function km_can_auto_bitrate_ext (ByVal komodo As Long, ByVal channel As Long, ByVal num_bitrates_hz As Long, ByRef bitrates_hz() As Long) as Long
    If check_version() Then
        Dim bitrates_hz_num_bitrates_hz As Long
        bitrates_hz_num_bitrates_hz = tp_min(num_bitrates_hz, UBound(bitrates_hz) - LBound(bitrates_hz) + 1)
        km_can_auto_bitrate_ext = vb6_km_can_auto_bitrate_ext(komodo, channel, bitrates_hz_num_bitrates_hz, bitrates_hz(0))
    Else
        km_can_auto_bitrate_ext = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Get the sample rate in hertz.

Public Function km_get_samplerate (ByVal komodo As Long) as Long
    If check_version() Then
        km_get_samplerate = vb6_km_get_samplerate(komodo)
    Else
        km_get_samplerate = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Configure the target power.  Returns power status or error code.
' Set the target power for the specified CAN channel.

Public Function km_can_target_power (ByVal komodo As Long, ByVal channel As Long, ByVal power As Long) as Long
    If check_version() Then
        km_can_target_power = vb6_km_can_target_power(komodo, channel, power)
    Else
        km_can_target_power = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Enable the Komodo.

Public Function km_enable (ByVal komodo As Long) as Long
    If check_version() Then
        km_enable = vb6_km_enable(komodo)
    Else
        km_enable = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Disable the Komodo.

Public Function km_disable (ByVal komodo As Long) as Long
    If check_version() Then
        km_disable = vb6_km_disable(komodo)
    Else
        km_disable = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Status mask for km_can_read
' GPIO Configuration
' GPIO Mask
' Event mask for km_can_read
' Read a single CAN packet from the Komodo data stream.
' This will block for timeout_ms milliseconds; 0 will return
' immediately, and MAXINT will block indefinitely.
' timestamp is in units of nanoseconds.

Public Function km_can_read (ByVal komodo As Long, ByRef info As km_can_info_t, ByRef pkt As km_can_packet_t, ByVal num_bytes As Long, ByRef data() As Byte) as Long
    If check_version() Then
        Dim data_num_bytes As Long
        data_num_bytes = tp_min(num_bytes, UBound(data) - LBound(data) + 1)
        km_can_read = vb6_km_can_read(komodo, info, pkt, data_num_bytes, data(0))
    Else
        km_can_read = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Flags mask
' Submit a CAN packet to the Komodo data stream, asynchronously.

Public Function km_can_async_submit (ByVal komodo As Long, ByVal channel As Long, ByVal flags As Byte, ByRef pkt As km_can_packet_t, ByVal num_bytes As Long, ByRef data() As Byte) as Long
    If check_version() Then
        Dim data_num_bytes As Long
        data_num_bytes = tp_min(num_bytes, UBound(data) - LBound(data) + 1)
        km_can_async_submit = vb6_km_can_async_submit(komodo, channel, flags, pkt, data_num_bytes, data(0))
    Else
        km_can_async_submit = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Collect a response to a CAN packet submitted to the Komodo data
' stream, asynchronously.

Public Function km_can_async_collect (ByVal komodo As Long, ByVal timeout_ms As Long, ByRef arbitration_count As Long) as Long
    If check_version() Then
        km_can_async_collect = vb6_km_can_async_collect(komodo, timeout_ms, arbitration_count)
    Else
        km_can_async_collect = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Write a stream of bytes to the CAN slave device.  The return
' value of the function is a status code.

Public Function km_can_write (ByVal komodo As Long, ByVal channel As Long, ByVal flags As Byte, ByRef pkt As km_can_packet_t, ByVal num_bytes As Long, ByRef data() As Byte, ByRef arbitration_count As Long) as Long
    If check_version() Then
        Dim data_num_bytes As Long
        data_num_bytes = tp_min(num_bytes, UBound(data) - LBound(data) + 1)
        km_can_write = vb6_km_can_write(komodo, channel, flags, pkt, data_num_bytes, data(0), arbitration_count)
    Else
        km_can_write = KM_INCOMPATIBLE_LIBRARY
    End If
End Function



'==========================================================================
' GPIO API
'==========================================================================
' Enumeration of input GPIO pin bias configurations.
' Enumeration of input GPIO pin trigger edge condition.
' Configure a GPIO pin to act as an input.  The return value
' of the function is a status code

Public Function km_gpio_config_in (ByVal komodo As Long, ByVal pin_number As Byte, ByVal bias As Byte, ByVal trigger As Byte) as Long
    If check_version() Then
        km_gpio_config_in = vb6_km_gpio_config_in(komodo, pin_number, bias, trigger)
    Else
        km_gpio_config_in = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Enumeration of output GPIO pin driver configurations.
' Enumeration of output GPIO pin sources.
' Configure a GPIO pin to act as an output. The return value
' of the function is a status code.

Public Function km_gpio_config_out (ByVal komodo As Long, ByVal pin_number As Byte, ByVal drive As Byte, ByVal source As Byte) as Long
    If check_version() Then
        km_gpio_config_out = vb6_km_gpio_config_out(komodo, pin_number, drive, source)
    Else
        km_gpio_config_out = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Set the value of any GPIO pins configured as software controlled
' outputs. The return value of the function is a status code.

Public Function km_gpio_set (ByVal komodo As Long, ByVal value As Byte, ByVal mask As Byte) as Long
    If check_version() Then
        km_gpio_set = vb6_km_gpio_set(komodo, value, mask)
    Else
        km_gpio_set = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


' Returns the current values of all GPIO pins.

Public Function km_gpio_get (ByVal komodo As Long) as Long
    If check_version() Then
        km_gpio_get = vb6_km_gpio_get(komodo)
    Else
        km_gpio_get = KM_INCOMPATIBLE_LIBRARY
    End If
End Function


